#include "main.h"
#include "image.h"


bool find_probable_starting_point(ALLEGRO_BITMAP* bitmap, int &start_x, int &start_y)
{
	int width = al_get_bitmap_width(bitmap);
	int height = al_get_bitmap_height(bitmap);

	for (int y = 0; y < height - COLOR_TILE_SIZE; y++)
	{
		for (int x = 0; x < width - COLOR_TILE_SIZE; x++)
		{
			if (is_probable_starting_point(bitmap, y, x))
			{
				start_x = x;
				start_y = y;
				return true;
			}
		}
	}
	return false;
}


bool is_probable_starting_point(ALLEGRO_BITMAP* bitmap, int y, int x)
{
	// The first point from which sixteen consecutive diagonals have a single color
	// is probably the starting point of a palette. Good enough, right?

	int current_x;
	int current_y;

	for (int i = 0; i < 16; i++)
	{
		current_x = x + i * COLOR_TILE_SIZE;
		current_y = y + i * COLOR_TILE_SIZE;

		if (!is_single_color_diagonal(bitmap, current_x, current_y))
		{
			return false;
		}
	}
	return true;
}


bool is_single_color_diagonal(ALLEGRO_BITMAP* bitmap, int x, int y)
{
	ALLEGRO_COLOR orig_color = al_get_pixel(bitmap, x, y);
	ALLEGRO_COLOR new_color;

	for (int i = 0; i < COLOR_TILE_SIZE; i++)
	{
		new_color = al_get_pixel(bitmap, x, y);
		if (new_color.r != orig_color.r || new_color.g != orig_color.g || new_color.b != orig_color.b)
		{
			return false;
		}
		x++;
		y++;
	}
	return true;
}

