#include "main.h"

#include "image.h"


int main(int argc, char** argv)
{
	libcon_init(argc, argv, "[image]");

	if (!al_init() || !al_init_image_addon())
	{
		std::cerr << "Failed to initialize library." << std::endl;
		return 1;
	}

	std::string image_filename = libcon_require_filename("Screenshot of Lunar Magic's palette editor:");
	libcon_end();

	ALLEGRO_BITMAP* bitmap = al_load_bitmap(image_filename.c_str());
	if (!bitmap)
	{
		std::cerr << "Failed to open screenshot. (If the file name has spaces, put it in \"quotes\".)" << std::endl;
		return 1;
	}

	al_lock_bitmap(bitmap, ALLEGRO_PIXEL_FORMAT_ANY, ALLEGRO_LOCK_READONLY);

	int start_x;
	int start_y;
	if (!find_probable_starting_point(bitmap, start_x, start_y))
	{
		std::cerr << "Nothing in this screenshot looks like a palette from Lunar Magic." << std::endl;
		return 1;
	}

	std::vector<unsigned char> output;
	read_bitmap_into_output(bitmap, start_x, start_y, output);
	if (output.size() > OUTPUT_SIZE)
	{
		std::cerr << "The output is larger than it should be. Program aborted." << std::endl;
		return 1;
	}

	if (!generate_pal_file(image_filename + ".pal", output))
	{
		std::cerr << "Failed to create palette file." << std::endl;
		return 1;
	}
	std::cout << "Palette file created." << std::endl;

	al_destroy_bitmap(bitmap);
	libcon_pause();

	return 0;
}


bool generate_pal_file(std::string pal_filename, std::vector<unsigned char> &output)
{
	bool retval = true;

	std::ofstream pal_file;
	pal_file.open(pal_filename);
	if (pal_file.fail())
	{
		retval = false;
	}
	for (auto &value : output)
	{
		pal_file << value;
	}
	pal_file.close();

	return retval;
}


void read_bitmap_into_output(ALLEGRO_BITMAP* bitmap, int start_x, int start_y, std::vector<unsigned char> &output)
{
	ALLEGRO_COLOR color;
	for (int y = start_y; y < PALETTE_SIZE; y += 16)
	{
		for (int x = start_x; x < PALETTE_SIZE; x += 16)
		{
			color = al_get_pixel(bitmap, x, y);
			output.push_back((unsigned char)(color.r * 255));
			output.push_back((unsigned char)(color.g * 255));
			output.push_back((unsigned char)(color.b * 255));
		}
	}
}